#################################################################################
#
# (c) Copyright 2010 William Stein
#
#  This file is part of SAGECORE
#
#  SAGECORE is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
# 
#  SAGECORE is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
# 
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#################################################################################

DEBUG=True
CHECK=False

import os, shutil, pickle

cur_dir = os.path.abspath(os.path.curdir)

target = os.path.join(cur_dir, 'build', 'SAGE_ROOT')
if not os.path.exists(target):
    os.makedirs(target)

target_bin = os.path.join(target, 'local', 'bin')
if not os.path.exists(target_bin):
    os.makedirs(target_bin)
    
os.environ['SAGE_ROOT'] = target
os.environ['SAGE_LOCAL'] = os.path.join(target, 'local')

package_dir = os.path.join(cur_dir, 'packages')

if not os.path.exists('.build_db'):
    build_db = {}
else:
    build_db = pickle.load(open('.build_db'))

print build_db

def save_build_db():
    pickle.dump(build_db, open('.build_db','w'))

def build_package(name):
    if name in build_db:
        if build_db[name]['success']:
            print "Package %s already built."%name
            return
    else:
        build_db[name] = {}
        
    build_dir = os.path.join(cur_dir, 'build', 'tmp', name)
    if os.path.exists(build_dir):
        shutil.rmtree(build_dir)
    
    # copy source code for the package to build/tmp/
    print "Copying package %s to %s"%(name, build_dir)
    shutil.copytree(os.path.join(package_dir, name), build_dir)

    # run spkg-install
    cmd = 'cd "%s" && ./spkg-install'%build_dir
    print cmd
    status = os.system(cmd)
    if status != 0:
        build_db[name]['success'] = False
        raise RuntimeError, "Error building %s"%name

    build_db[name]['success'] = True    

    # run spkg-check
    if CHECK:
        print "spkg-check not implemented yet"
        build_db[name]['check'] = True            
        pass

    # skip doing this for now
    if not DEBUG:
        print "Delete source code for %s from build/tmp..."%name
        shutil.rmtree(build_dir)

packages = [
    'mpir',
    'readline',
    'pari',
    'ntl',
    'ratpoints',
    'cliquer',
    'eclib',
    'ecm',
    'flint',
    'mpfr',
    'mpfi',
    'libfplll',
    'givaro',
    'gsl',
    'iml',
    'lcalc',
    'libm4ri',
    'linbox',
    'pynac',
    'symmetrica',
    'zn_poly',
    'singular',
    'boost-cropped',
    'polybori',
    'libpng'
    ]

def build_all():
    for name in packages:
        build_package(name)

try:
    build_all()
finally:
    save_build_db()

